@@ -6,12 +6,15 @@
 
 import os
 import mimetypes
+import json
 import base64
+from pathlib import Path
 
 from traitlets import default, Unicode, Bool
 from traitlets.config import Config
 from jupyter_core.paths import jupyter_path
 import jinja2
+import markupsafe
 
 if tuple(int(x) for x in jinja2.__version__.split(".")[:3]) < (3, 0, 0):
     from jinja2 import contextfilter
@@ -27,6 +30,58 @@ from nbconvert.filters.markdown_mistune import IPythonRenderer, MarkdownWithMath
 from .templateexporter import TemplateExporter
 
 
+def find_lab_theme(theme_name):
+    """
+    Find a JupyterLab theme location by name.
+
+    Parameters
+    ----------
+    theme_name : str
+        The name of the labextension theme you want to find.
+
+    Raises
+    ------
+    ValueError
+        If the theme was not found, or if it was not specific enough.
+
+    Returns
+    -------
+    theme_name: str
+        Full theme name (with scope, if any)
+    labextension_path : Path
+        The path to the found labextension on the system.
+    """
+    paths = jupyter_path('labextensions')
+
+    matching_themes = []
+    theme_path = None
+    for path in paths:
+        for (dirpath, dirnames, filenames) in os.walk(path):
+            # If it's a federated labextension that contains themes
+            if 'package.json' in filenames and 'themes' in dirnames:
+                # TODO Find the theme name in the JS code instead?
+                # TODO Find if it's a light or dark theme?
+                with open(Path(dirpath) / 'package.json', 'r', encoding="utf-8") as fobj:
+                    labext_name = json.loads(fobj.read())['name']
+
+                if labext_name == theme_name or theme_name in labext_name.split('/'):
+                    matching_themes.append(labext_name)
+
+                    full_theme_name = labext_name
+                    theme_path = Path(dirpath) / 'themes' / labext_name
+
+    if len(matching_themes) == 0:
+        raise ValueError(f'Could not find lab theme "{theme_name}"')
+
+    if len(matching_themes) > 1:
+        raise ValueError(
+            f'Found multiple themes matching "{theme_name}": {matching_themes}. '
+            'Please be more specific about which theme you want to use.'
+        )
+
+    return full_theme_name, theme_path
+
+
 class HTMLExporter(TemplateExporter):
     """
     Exports a basic HTML document.  This exporter assists with the export of
@@ -89,10 +144,14 @@ class HTMLExporter(TemplateExporter):
     def _template_name_default(self):
         return 'lab'
 
-    theme = Unicode('light',
-                    help='Template specific theme(e.g. the JupyterLab CSS theme for the lab template)'
+    theme = Unicode(
+        'light',
+        help="Template specific theme(e.g. the name of a JupyterLab CSS theme distributed as prebuilt extension for the lab template)"
     ).tag(config=True)
 
+    embed_images = Bool(False,
+        help="Whether or not to embed images as base64 in markdown cells.").tag(config=True)
+
     output_mimetype = 'text/html'
 
     @property
@@ -123,9 +182,16 @@ class HTMLExporter(TemplateExporter):
         """Markdown to HTML filter respecting the anchor_link_text setting"""
         cell = context.get('cell', {})
         attachments = cell.get('attachments', {})
-        renderer = IPythonRenderer(escape=False, attachments=attachments,
-                                   anchor_link_text=self.anchor_link_text,
-                                   exclude_anchor_links=self.exclude_anchor_links)
+        path = context.get('resources', {}).get('metadata', {}).get('path', '')
+
+        renderer = IPythonRenderer(
+            escape=False,
+            attachments=attachments,
+            embed_images=self.embed_images,
+            path=path,
+            anchor_link_text=self.anchor_link_text,
+            exclude_anchor_links=self.exclude_anchor_links
+        )
         return MarkdownWithMath(renderer=renderer).render(source)
 
     def default_filters(self):
@@ -148,12 +214,36 @@ class HTMLExporter(TemplateExporter):
         def resources_include_css(name):
             env = self.environment
             code = """<style type="text/css">\n%s</style>""" % (env.loader.get_source(env, name)[0])
-            return jinja2.Markup(code)
+            return markupsafe.Markup(code)
+
+        def resources_include_lab_theme(name):
+            # Try to find the theme with the given name, looking through the labextensions
+            _, theme_path = find_lab_theme(name)
+
+            with open(theme_path / 'index.css', 'r') as file:
+                data = file.read()
+
+            # Embed assets (fonts, images...)
+            for asset in os.listdir(theme_path):
+                local_url = "url({})".format(Path(asset).as_posix())
+
+                if local_url in data:
+                    mime_type = mimetypes.guess_type(asset)[0]
+
+                    # Replace asset url by a base64 dataurl
+                    with open(theme_path / asset, 'rb') as assetfile:
+                        base64_data = base64.b64encode(assetfile.read())
+                        base64_data = base64_data.replace(b'\n', b'').decode('ascii')
+
+                        data = data.replace(local_url, 'url(data:{};base64,{})'.format(mime_type, base64_data))
+
+            code = """<style type="text/css">\n%s</style>""" % data
+            return markupsafe.Markup(code)
 
         def resources_include_js(name):
             env = self.environment
             code = """<script>\n%s</script>""" % (env.loader.get_source(env, name)[0])
-            return jinja2.Markup(code)
+            return markupsafe.Markup(code)
 
         def resources_include_url(name):
             env = self.environment
@@ -173,14 +263,16 @@ class HTMLExporter(TemplateExporter):
                             data = f.read()
                             break
                 else:
-                    raise ValueError("No file %r found in %r" % (name, searchpaths))
+                    raise ValueError("No file %r found in %r" % (name, searchpath))
             data = base64.b64encode(data)
             data = data.replace(b'\n', b'').decode('ascii')
             src = 'data:{mime_type};base64,{data}'.format(mime_type=mime_type, data=data)
-            return jinja2.Markup(src)
+            return markupsafe.Markup(src)
+
         resources = super()._init_resources(resources)
         resources['theme'] = self.theme
         resources['include_css'] = resources_include_css
+        resources['include_lab_theme'] = resources_include_lab_theme
         resources['include_js'] = resources_include_js
         resources['include_url'] = resources_include_url
         resources['require_js_url'] = self.require_js_url
