@@ -10,7 +10,7 @@ import json
 
 from traitlets import default
 from traitlets.config import Config
-from jinja2 import DictLoader, TemplateNotFound
+from jinja2 import TemplateNotFound
 from nbformat import v4
 from unittest.mock import patch
 from concurrent.futures import ProcessPoolExecutor
@@ -636,4 +636,4 @@ class TestExporter(ExportersTestsBase):
 
     def _make_exporter(self, config=None):
         exporter = SampleExporter(config=config)
-        return exporter
\ No newline at end of file
+        return exporter
